module ISTEX where

import Data.ByteString qualified as B
import Data.Default.Class (def)
import Data.Text (Text)
import Data.X509.CertificateStore (CertificateStore)
import ISTEX.Client
import Network.Connection (TLSSettings(..))
import Network.HTTP.Client (ManagerSettings, newManager)
import Network.HTTP.Client.TLS (mkManagerSettings)
import Network.TLS (ClientParams(..), EMSMode(AllowEMS), Supported(..), defaultParamsClient, sharedCAStore)
import Servant.Client
import System.X509 (getSystemCertificateStore)


runIstexAPIClient :: ClientM (Documents) -> IO (Either ClientError Documents)
runIstexAPIClient cmd = do
  scs <- getSystemCertificateStore
  manager' <- newManager $ getTLSManagerSettings scs
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.istex.fr" 443 "document")

-- | https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/456
getTLSManagerSettings :: CertificateStore -> ManagerSettings
getTLSManagerSettings scs = mkManagerSettings tlsSettings Nothing
  where
    tlsSettings = TLSSettings clientParams
    -- See Network.Connection code on how `TLSSettings` is handled to make client params.
    -- We need to change EMS and also apply system certificates, as they aren't in the defaults.
    clientParams = (defaultParamsClient "" B.empty)
      { clientSupported = supported
      , clientShared    = def
        { sharedCAStore = scs }
      }
    supported = def { supportedExtendedMainSecret = AllowEMS }

getMetadataWith :: Text -> Maybe Int -> IO (Either ClientError Documents)
getMetadataWith q n = do
  runIstexAPIClient $
    search
    (Just "author,title,abstract,publicationDate,host")
    n
    (Just q)

type Progress = ScrollResponse -> IO ()
type ErrorHandler = ClientError -> IO ()

runIstexScrollAPIClient :: ClientM (ScrollResponse) -> IO (Either ClientError ScrollResponse)
runIstexScrollAPIClient cmd = do
  scs <- getSystemCertificateStore
  manager' <- newManager $ getTLSManagerSettings scs
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.istex.fr" 443 "document")

maxScrollPages :: Int
maxScrollPages = 1000

getMetadataScroll :: Text -> Text -> Maybe Text -> Int -> IO (Either ClientError Documents)
getMetadataScroll q scroll scrollId page = do
  if page >= maxScrollPages then
    pure $ Right mempty
  else do
    -- Prelude.putStrLn $ "[getMetadataScroll] scroll: " <> show scroll
    -- Prelude.putStrLn $ "[getMetadataScroll] scrollId: " <> show scrollId
    -- Prelude.putStrLn $ "[getMetadataScroll] page: " <> show page
    eRes <- runIstexScrollAPIClient $
      searchScroll
      (Just "author,title,abstract,publicationDate,host")
      scroll
      (Just q)
      scrollId
    case eRes of
      Left err -> pure $ Left err
      Right _res@(ScrollResponse { .. }) -> do
        eDocs <- getMetadataScroll q scroll _scroll_scrollId (page + 1)
        case eDocs of
          Left err -> pure $ Left err
          Right docs -> pure $ Right $ _scroll_documents <> docs
        --mappend _scroll_documents <*> getMetadataScroll q scroll (Just _scroll_scrollId) (page + 1)

getMetadataScrollProgress :: Text -> Text -> Maybe Text -> Progress -> ErrorHandler -> IO ()
getMetadataScrollProgress q scroll scrollId progress errorHandler = do
  eRes <- runIstexScrollAPIClient $
    searchScroll
    (Just "author,title,abstract,publicationDate,host")
    scroll
    (Just q)
    scrollId
  case eRes of
    Left err -> errorHandler err
    Right res@(ScrollResponse { _scroll_noMoreScrollResults
                              , _scroll_nextScrollURI
                              , _scroll_scrollId }) -> do
      progress res
      getMetadataScrollProgress q scroll _scroll_scrollId progress errorHandler

type Query = Text

runTest :: Query -> Maybe Int -> IO ()
runTest query n = do
  res <- getMetadataWith query n
  case res of
    (Left err)  -> print $ show err
    (Right val) -> do
      print val


